<html>
<h2>{% i18ncp "The total number of people displayed" "%1 person" "%1 people" contacts|length %}</h2>

{% if contacts %}
<table border="1" cellpadding="2">
<tr>
  <th>{% i18nc "The name of a person" "Name" %}</th>
  <th>{% i18n "Email" %}</th>
  <th>{% i18n "Phone" %}</th>
  <th>{{ _("Nickname") }}</th>
  <th>{% i18n "Address" %}</th>
  <th>{% i18n "Birthday" %}</th>
  <th>{% i18n "Salary" %}</th>
  <th>{% i18n "Rating" %}</th>
</tr>
{% for contact in contacts %}
<tr>
  <td nowrap>{{ contact.name }}</td>
  <td>{{ contact.email }}</td>
  <td>{{ contact.phone }}</td>
  <td nowrap>{{ contact.nickname }}</td>
  <td nowrap>{{ contact.address.houseNumber }} {{ contact.address.streetName }}<br/>{{ contact.address.city }}</td>
  <td>{{ _(contact.birthday) }}</td>
  <td align="right" nowrap>{% l10n_money contact.salary contact.salaryCurrency %}</td>
  <td align="right" nowrap>{{ _(contact.rating) }}</td>
</tr>
{% endfor %}
</table>
{% endif %}

</html>